Extensions
==========


Extensions are enabled per-plugin. A plugin can request an extension by providing
it in its manifest. If the extension is available, it will be activated for the
plugin:

    ...
    "extensions": [ "config" ],
    ...

To see if an extension is available, your instance must make use of the 
`PluginAwareInterface` in order to get the `PluginInstance`:

    class MyPlugin implements PluginAwareInterface {

        protected $plugin;

        protected $config;

        public function setPluginInstance(PluginInstance $plugin) {
            $this->plugin = $plugin;
        }

        public function onLoad()
        {
            if ($this->plugin->hasExtension("config")) {
                $this->config = $this->plugin->getConfiguration();
            }
        }

        public function onUnload()
        {
            // required by interface, called on exit or reload
        }

    }
    


## Resources

*NOT IMPLEMENTED*

Via stream wrapper:

    $banner = file_get_contents("pluginres://my.plugin.name/banner");

## Configuration

*PARTIALLY IMPLEMENTED*

    $config = $plugin->getConfiguration();
    $config->addStringProp("color");

## Data

*NOT IMPLEMENTED*

The `data` extension assigns a directory for plugin data. This can be accessed via
the plugin instance, like so:

    $root = $plugin->getDataRoot();
    $filename = $plugin->getDataPath("names.db");

Or it can be used via the `plugindata://` stream wrapper:

    $filename = "plugindata://my.plugin.name/names.db";


## Symfony

*IMPLEMENTED*

By enabling the Symfony extension, you can integrate services and classes into an
application straight from json and make your plugin instance container aware.

The extension need to know the application instance and the container in the constructor:

    // $pm is the plugin manager
    // $app is the symfony application
    // $container is the container
    $pm->addExtension(new SymfonyExtension($app, $container));

If a `services.json` file resides in the root of the plugin, it will be parsed. The
definitions will be created and and injected into the service container of the 
application.

    {
        "my.service.name": {
            "class": "Namespace\\Of\\ClassName",
            "arguments": []
        }
    }

Likewise, if a `commands.json` files is found in the root it will be parsed and commands
will be added to the application.

    {
        "group:command": "Class\\Of\\The\\Command"
    }

