<?php

namespace NoccyLabs\Plugin\Extension;

use NoccyLabs\Plugin\PluginManager;
use NoccyLabs\Plugin\PluginInstance;

class ExtensionManager
{
    protected $extensions = [];
    
    protected $pluginManager;

    public function __construct(PluginManager $pluginManager)
    {
        $this->pluginManager = $pluginManager;

        if (!defined("PLUGIN_NO_DEFAULT_EXTENSIONS")) {
            $this->extensions = [
                "resource"  => new ResourceExtension(),
                "config"    => new ConfigurationExtension(),
            ];
        }

    }

    public function addExtension(AbstractExtension $extension, $name=null)
    {
        if (!$name) {
            $name = $extension->getDefaultName();
        }
        $this->extensions[$name] = $extension;
    }

    public function createExtension(PluginInstance $plugin, $extension)
    {
        if (!array_key_exists($extension, $this->extensions)) {
            return false;
        }
        $ext = clone $this->extensions[$extension];
        $ext->setPluginManager($this->pluginManager);
        $ext->setPluginInstance($plugin);
        return $ext;
    }
}

